const defaultRelevantStyles = [
    'font',
    'fontFamily',
    'fontSize',
    'fontWeight',
    'fontStyle',
    'fontVariant',
    'lineHeight',
];
export class TextareaSelectionBounds {
    constructor(textArea, options) {
        var _a, _b;
        this._textArea = textArea;
        this._options = {
            relevantStyles: (_a = options === null || options === void 0 ? void 0 : options.relevantStyles) !== null && _a !== void 0 ? _a : [],
            textAreaPadding: (_b = options === null || options === void 0 ? void 0 : options.textAreaPadding) !== null && _b !== void 0 ? _b : { top: 0, left: 0 },
        };
    }
    get relevantStyles() {
        return [...defaultRelevantStyles, ...this._options.relevantStyles];
    }
    getCurrentSelection() {
        return {
            from: this._textArea.selectionStart,
            to: this._textArea.selectionEnd,
        };
    }
    getBounds() {
        const selection = this.getCurrentSelection();
        return this.getBoundsForSelection(selection);
    }
    getBoundsForSelection(selection) {
        const actualFrom = Math.min(selection.from, selection.to);
        const actualTo = Math.max(selection.from, selection.to);
        const amountOfScrollY = this._textArea.scrollTop;
        const amountOfScrollX = this._textArea.scrollLeft;
        const div = document.createElement('div');
        const copyStyle = getComputedStyle(this._textArea);
        for (const prop of this.relevantStyles) {
            div.style[prop] = copyStyle[prop];
        }
        div.style.whiteSpace = 'pre-wrap';
        div.style.width = `${this._textArea.scrollWidth}px`;
        div.style.height = 'auto';
        const textContentUntilSelection = this._textArea.value.substring(0, actualFrom);
        const textContentSelection = this._textArea.value.substring(actualFrom, actualTo);
        const spanZeroWidth = document.createElement('span');
        spanZeroWidth.textContent = '\u200B';
        const spanZeroWidth2 = document.createElement('span');
        spanZeroWidth2.textContent = '\u200B';
        const spanUntilSelection = document.createElement('span');
        spanUntilSelection.textContent = textContentUntilSelection;
        const spanSelection = document.createElement('span');
        spanSelection.textContent = textContentSelection;
        div.appendChild(spanUntilSelection);
        div.appendChild(spanZeroWidth);
        div.appendChild(spanSelection);
        div.appendChild(spanZeroWidth2);
        document.body.appendChild(div);
        const divTop = div.getBoundingClientRect().top;
        const divLeft = div.getBoundingClientRect().left;
        const top = spanZeroWidth.getBoundingClientRect().top -
            divTop -
            amountOfScrollY +
            this._options.textAreaPadding.top;
        const left = spanSelection.getBoundingClientRect().left -
            divLeft -
            amountOfScrollX +
            this._options.textAreaPadding.left;
        const height = spanSelection.getBoundingClientRect().height;
        const width = spanSelection.getBoundingClientRect().width;
        document.body.removeChild(div);
        return {
            top,
            left,
            height,
            width,
        };
    }
}
//# sourceMappingURL=textarea-selection-bounds.js.map